@@ -4,10 +4,11 @@ package JSON::Schema::Modern::Error;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single error from a JSON Schema evaluation
 
-our $VERSION = '0.581';
+our $VERSION = '0.583';
 
 use 5.020;
 use Moo;
+with 'JSON::Schema::Modern::ResultNode';
 use strictures 2;
 use stable 0.031 'postderef';
 use experimental 'signatures';
@@ -15,11 +16,9 @@ use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use Safe::Isa;
-use JSON::PP ();
 use MooX::TypeTiny;
-use Types::Standard qw(Str Bool Undef InstanceOf Enum Tuple);
-use Types::Common::Numeric qw(PositiveInt PositiveOrZeroInt);
+use Types::Standard qw(Str Bool Enum Tuple);
+use Types::Common::Numeric qw(PositiveInt);
 use namespace::clean;
 
 use overload
@@ -27,28 +26,12 @@ use overload
   '""' => sub { $_[0]->stringify },
   fallback => 1;
 
-has [qw(
-  instance_location
-  keyword_location
-  error
-)] => (
+has error => (
   is => 'ro',
   isa => Str,
   required => 1,
 );
 
-has absolute_keyword_location => (
-  is => 'ro',
-  isa => InstanceOf['Mojo::URL'],
-  coerce => sub { $_[0]->$_isa('Mojo::URL') ? $_[0] : Mojo::URL->new($_[0]) },
-);
-
-has keyword => (
-  is => 'ro',
-  isa => Str|Undef,
-  required => 1,
-);
-
 has exception => (
   is => 'ro',
   isa => Bool,
@@ -64,40 +47,12 @@ has recommended_response => (
   isa => Tuple[PositiveInt, Str],
 );
 
-has depth => (
-  is => 'ro',
-  isa => PositiveOrZeroInt,
-  required => 1,
-);
-
-sub TO_JSON ($self) {
-  return +{
-    # note that locations are JSON pointers, not uri fragments!
-    instanceLocation => $self->instance_location,
-    keywordLocation => $self->keyword_location,
-    !defined($self->absolute_keyword_location) ? ()
-      : ( absoluteKeywordLocation => $self->absolute_keyword_location->to_string ),
-    error => $self->error,  # TODO: allow localization
-  };
-}
-
 sub stringify ($self) {
   ($self->mode//'evaluate') eq 'traverse'
     ? '\''.$self->keyword_location.'\': '.$self->error
     : '\''.$self->instance_location.'\': '.$self->error;
 }
 
-sub dump ($self) {
-  my $encoder = JSON::Schema::Modern::_JSON_BACKEND()->new
-    ->utf8(0)
-    ->convert_blessed(1)
-    ->canonical(1)
-    ->indent(1)
-    ->space_after(1);
-  $encoder->indent_length(2) if $encoder->can('indent_length');
-  $encoder->encode($self);
-}
-
 1;
 
 __END__
@@ -114,7 +69,7 @@ JSON::Schema::Modern::Error - Contains a single error from a JSON Schema evaluat
 
 =head1 VERSION
 
-version 0.581
+version 0.583
 
 =head1 SYNOPSIS
 
@@ -181,7 +136,7 @@ construct a tree-like structure of errors.
 
 =head1 METHODS
 
-=for Pod::Coverage stringify mode
+=for Pod::Coverage stringify mode BUILDARGS
 
 =head2 TO_JSON
 
@@ -203,6 +158,8 @@ Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Mo
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
+=for stopwords OpenAPI
+
 You can also find me on the L<JSON Schema Slack server|https://json-schema.slack.com> and L<OpenAPI Slack
 server|https://open-api.slack.com>, which are also great resources for finding help.
 
