@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Modern
 
-our $VERSION = '0.581';
+our $VERSION = '0.583';
 
 use 5.020;
 use strictures 2;
@@ -20,8 +20,6 @@ use Ref::Util 0.100 qw(is_ref is_plain_arrayref is_plain_hashref);
 use Scalar::Util qw(blessed looks_like_number);
 use Storable 'dclone';
 use Feature::Compat::Try;
-use JSON::Schema::Modern::Error;
-use JSON::Schema::Modern::Annotation;
 use namespace::clean;
 
 use Exporter 'import';
@@ -29,6 +27,7 @@ use Exporter 'import';
 our @EXPORT_OK = qw(
   is_type
   get_type
+  is_bignum
   is_equal
   is_elements_unique
   jsonp
@@ -76,12 +75,12 @@ sub is_type ($type, $value) {
     }
 
     if ($type eq 'number') {
-      return ref($value) =~ /^Math::Big(?:Int|Float)$/
+      return is_bignum($value)
         || !($flags & B::SVf_POK) && ($flags & (B::SVf_IOK | B::SVf_NOK));
     }
 
     if ($type eq 'integer') {
-      return ref($value) =~ /^Math::Big(?:Int|Float)$/ && $value->is_int
+      return is_bignum($value) && $value->is_int
         || !($flags & B::SVf_POK) && ($flags & (B::SVf_IOK | B::SVf_NOK)) && int($value) == $value;
     }
   }
@@ -100,7 +99,7 @@ sub get_type ($value) {
   return 'null' if not defined $value;
   return 'array' if is_plain_arrayref($value);
 
-  return ref($value) =~ /^Math::Big(?:Int|Float)$/ ? ($value->is_int ? 'integer' : 'number')
+  return is_bignum($value) ? ($value->is_int ? 'integer' : 'number')
       : (blessed($value) ? '' : 'reference to ').ref($value)
     if is_ref($value);
 
@@ -120,6 +119,10 @@ sub is_bool ($value) {
       or $value->isa('JSON::XS::Boolean'));
 }
 
+sub is_bignum ($value) {
+  ref($value) =~ /^Math::Big(?:Int|Float)$/;
+}
+
 # compares two arbitrary data payloads for equality, as per
 # https://json-schema.org/draft/2020-12/json-schema-core.html#rfc.section.4.2.2
 # if provided with a state hashref with a 'path' key, any differences are recorded within
@@ -184,10 +187,10 @@ sub is_elements_unique ($array, $equal_indices = undef, $state = {}) {
 }
 
 # shorthand for creating and appending json pointers
-# the first argument is a a json pointer; remaining arguments are path segments to be encoded and
+# the first argument is a json pointer; remaining arguments are path segments to be encoded and
 # appended
 sub jsonp {
-  return join('/', shift, map s/~/~0/gr =~ s!/!~1!gr, map +(is_plain_arrayref($_) ? @$_ : $_), grep defined, @_);
+  return join('/', shift, map s/~/~0/gr =~ s!/!~1!gr, grep defined, @_);
 }
 
 # splits a json pointer apart into its path segments
@@ -202,10 +205,9 @@ sub local_annotations ($state) {
 }
 
 # shorthand for finding the canonical uri of the present schema location
-# last argument can be an arrayref, usually coming from $state->{_schema_path_suffix}
+# ensure that this code is kept consistent with the absolute_keyword_location builder in ResultNode
 sub canonical_uri ($state, @extra_path) {
   return $state->{initial_schema_uri} if not @extra_path and not length($state->{schema_path});
-  splice(@extra_path, -1, 1, $extra_path[-1]->@*) if @extra_path and is_plain_arrayref($extra_path[-1]);
   my $uri = $state->{initial_schema_uri}->clone;
   my $fragment = ($uri->fragment//'').(@extra_path ? jsonp($state->{schema_path}, @extra_path) : $state->{schema_path});
   undef $fragment if not length($fragment);
@@ -229,21 +231,24 @@ sub E ($state, $error_string, @args) {
   croak 'E called in void context' if not defined wantarray;
 
   # sometimes the keyword shouldn't be at the very end of the schema path
-  my $uri = canonical_uri($state, $state->{keyword}, $state->{_schema_path_suffix})
-    ->to_abs($state->{effective_base_uri});
+  my $sps = delete $state->{_schema_path_suffix};
+  my @schema_path_suffix = defined $sps && is_plain_arrayref($sps) ? $sps->@* : $sps//();
+
+  # we store the absolute uri in unresolved form until needed,
+  # and perform the rest of the calculations later.
+  my $uri = [ $state->{initial_schema_uri}, $state->{schema_path}, $state->{keyword}, @schema_path_suffix, $state->{effective_base_uri} ];
 
   my $keyword_location = $state->{traversed_schema_path}
-    .jsonp($state->{schema_path}, $state->{keyword}, delete $state->{_schema_path_suffix});
-
-  undef $uri if $uri eq '' and $keyword_location eq ''
-    or ($uri->fragment // '') eq $keyword_location and $uri->clone->fragment(undef) eq '';
+    .jsonp($state->{schema_path}, $state->{keyword}, @schema_path_suffix);
 
+  require JSON::Schema::Modern::Error;
   push $state->{errors}->@*, JSON::Schema::Modern::Error->new(
     depth => $state->{depth} // 0,
     keyword => $state->{keyword},
     instance_location => $state->{data_path},
     keyword_location => $keyword_location,
-    defined $uri ? ( absolute_keyword_location => $uri ) : (),
+    # we calculate absolute_keyword_location when instantiating the Error object for Result
+    _uri => $uri,
     error => @args ? sprintf($error_string, @args) : $error_string,
     $state->{exception} ? ( exception => $state->{exception} ) : (),
     $state->{recommended_response} ? ( recommended_response => $state->{recommended_response} ) : (),
@@ -270,12 +275,10 @@ sub A ($state, $annotation) {
 
   # we store the absolute uri in unresolved form until needed,
   # and perform the rest of the calculations later.
-
-  my $uri = [ canonical_uri($state, $state->{keyword}, $state->{_schema_path_suffix}),
-    $state->{effective_base_uri} ];
+  my $uri = [ $state->{initial_schema_uri}, $state->{schema_path}, $state->{keyword}, $state->{effective_base_uri} ];
 
   my $keyword_location = $state->{traversed_schema_path}
-    .jsonp($state->{schema_path}, $state->{keyword}, delete $state->{_schema_path_suffix});
+    .jsonp($state->{schema_path}, $state->{keyword});
 
   push $state->{annotations}->@*, {
     depth => $state->{depth} // 0,
@@ -340,7 +343,8 @@ sub assert_uri_reference ($state, $schema) {
   return 1;
 }
 
-# this is only suitable for checking URIs within schemas themselves
+# this is only suitable for checking URIs within schemas themselves,
+# which have fragments consisting of plain names (anchors) or json pointers
 sub assert_uri ($state, $schema, $override = undef) {
   croak 'assert_uri called in void context' if not defined wantarray;
 
@@ -361,6 +365,7 @@ sub assert_uri ($state, $schema, $override = undef) {
 }
 
 # produces an annotation whose value is the same as that of the current keyword
+# makes a copy as this is passed back to the user, who cannot be trusted to not mutate it
 sub annotate_self ($state, $schema) {
   A($state, is_ref($schema->{$state->{keyword}}) ? dclone($schema->{$state->{keyword}})
     : $schema->{$state->{keyword}});
@@ -368,7 +373,7 @@ sub annotate_self ($state, $schema) {
 
 sub sprintf_num ($value) {
   # use original value as stored in the NV, without losing precision
-  ref($value) =~ /^Math::Big(?:Int|Float)$/ ? $value->bstr : sprintf('%s', $value);
+  is_bignum($value) ? $value->bstr : sprintf('%s', $value);
 }
 
 1;
@@ -385,7 +390,7 @@ JSON::Schema::Modern::Utilities - Internal utilities for JSON::Schema::Modern
 
 =head1 VERSION
 
-version 0.581
+version 0.583
 
 =head1 SYNOPSIS
 
@@ -395,18 +400,18 @@ version 0.581
 
 This class contains internal utilities to be used by L<JSON::Schema::Modern>.
 
-=for Pod::Coverage is_type get_type is_bool is_equal is_elements_unique jsonp unjsonp local_annotations
+=for Pod::Coverage is_type get_type is_bignum is_bool is_equal is_elements_unique jsonp unjsonp local_annotations
 canonical_uri E A abort assert_keyword_exists assert_keyword_type assert_pattern assert_uri_reference assert_uri
 annotate_self sprintf_num
 
-=for stopwords OpenAPI
-
 =head1 SUPPORT
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Modern/issues>.
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
+=for stopwords OpenAPI
+
 You can also find me on the L<JSON Schema Slack server|https://json-schema.slack.com> and L<OpenAPI Slack
 server|https://open-api.slack.com>, which are also great resources for finding help.
 
