@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Validation;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Validation vocabulary
 
-our $VERSION = '0.581';
+our $VERSION = '0.583';
 
 use 5.020;
 use Moo;
@@ -19,7 +19,7 @@ use List::Util 'any';
 use Ref::Util 0.100 'is_plain_arrayref';
 use Scalar::Util 'looks_like_number';
 use if "$]" >= 5.022, POSIX => 'isinf';
-use JSON::Schema::Modern::Utilities qw(is_type get_type is_equal is_elements_unique E assert_keyword_type assert_pattern jsonp sprintf_num);
+use JSON::Schema::Modern::Utilities qw(is_type get_type is_bignum is_equal is_elements_unique E assert_keyword_type assert_pattern jsonp sprintf_num);
 use Math::BigFloat;
 use namespace::clean;
 
@@ -116,11 +116,11 @@ sub _eval_keyword_multipleOf ($class, $data, $schema, $state) {
       and do { $data = 0+$data; 1 });
 
   # if either value is a float, use the bignum library for the calculation for an accurate remainder
-  if (ref($data) =~ /^Math::Big(?:Int|Float)$/
-      or ref($schema->{multipleOf}) =~ /^Math::Big(?:Int|Float)$/
+  if (is_bignum($data) or is_bignum($schema->{multipleOf})
       or get_type($data) eq 'number' or get_type($schema->{multipleOf}) eq 'number') {
-    $data = ref($data) =~ /^Math::Big(?:Int|Float)$/ ? $data->copy : Math::BigFloat->new($data);
-    my $divisor = ref($schema->{multipleOf}) =~ /^Math::Big(?:Int|Float)$/ ? $schema->{multipleOf} : Math::BigFloat->new($schema->{multipleOf});
+    $data = is_bignum($data) ? $data->copy : Math::BigFloat->new($data);
+    my $divisor = is_bignum($schema->{multipleOf}) ? $schema->{multipleOf} : Math::BigFloat->new($schema->{multipleOf});
+
     my ($quotient, $remainder) = $data->bdiv($divisor);
     return E($state, 'overflow while calculating quotient') if $quotient->is_inf;
     return 1 if $remainder == 0;
@@ -331,7 +331,7 @@ JSON::Schema::Modern::Vocabulary::Validation - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.581
+version 0.583
 
 =head1 DESCRIPTION
 
@@ -357,14 +357,14 @@ the equivalent Draft 7 keywords that correspond to this vocabulary and are forma
 
 =back
 
-=for stopwords OpenAPI
-
 =head1 SUPPORT
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Modern/issues>.
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
+=for stopwords OpenAPI
+
 You can also find me on the L<JSON Schema Slack server|https://json-schema.slack.com> and L<OpenAPI Slack
 server|https://open-api.slack.com>, which are also great resources for finding help.
 
